indexing

   description: "Class that covers POSIX memory mapping."

   author: "Berend de Boer"
   date: "$Date: 2001/12/27 $"
   revision: "$Revision: #1 $"

class 

   POSIX_MEMORY_MAP

inherit
   
   POSIX_BASE
   
   PAPI_MMAN
   
   MEMORY
      redefine
         dispose
      end
   
   
creation
   
   make,
   make_private,
   make_shared
   
   
feature -- creation
   
   make (a_fd: POSIX_FILE_DESCRIPTOR; a_offset, a_size: INTEGER;  a_base: POINTER; a_prot, a_flags: INTEGER) is
         -- raw interface to mmap
      do
         fd := a_fd
         offset := a_offset
         size := a_size
         base := posix_mmap (a_base, size, a_prot, a_flags, fd.value, offset)
         if base = MAP_FAILED then
            raise_posix_error
         end
         create mem.make_from_pointer (base, a_size, False)
      end
   
   make_private (a_fd: POSIX_FILE_DESCRIPTOR; a_offset, a_size: INTEGER ) is
         -- make a mapping where changes are private
         -- this function can fail on certain system (Linux for
         -- example) if a_offset is not a multiple of PAGE_SIZE
      do
         make (a_fd, a_offset, a_size, default_pointer, PROT_READ + PROT_WRITE, MAP_PRIVATE)
      end
   
   make_shared (a_fd: POSIX_FILE_DESCRIPTOR; a_offset, a_size: INTEGER ) is
         -- make a mapping where changes are shared, i.e. the
         -- underlying object is also changed.
         -- this function can fail on certain system (Linux for
         -- example) if a_offset is not a multiple of PAGE_SIZE
      do
         make (a_fd, a_offset, a_size, default_pointer, PROT_READ + PROT_WRITE, MAP_SHARED)
      end
   
   
feature -- cleanup

   dispose is
      do
         if base /= default_pointer then
            close
         end
      rescue
         base := default_pointer
      end
   
   
feature -- unmap
   
   close is
         -- remove the mapping
      do
         safe_call (posix_munmap (base, size))
         base := default_pointer
         size := 0
         offset := 0
      end
   
  
feature -- state

   mem: STDC_BUFFER
         -- window to memory
   
   offset: INTEGER
         -- offset from file
   
   size: INTEGER
         -- number of bytes mapping

   fd: POSIX_FILE_DESCRIPTOR
   
   
feature {NONE} -- private state
   
   base: POINTER
   

end -- class POSIX_MEMORY_MAP
