indexing

   description: "Class that describes service names, i.e. an entry %
   %in /etc/services."

   author: "Berend de Boer"
   date: "$Date: 2001/12/27 $"
   revision: "$Revision: #1 $"


class

   SUS_SERVICE


inherit

   SUS_BASE

   SAPI_NETDB

   SAPI_IN


creation

   make_from_name,
   make_from_port


feature -- creation

   make_from_name (a_name, a_protocol: STRING) is
      require
         valid_name: a_name /= Void and then not a_name.is_empty
      local
         p: POINTER
      do
         p := posix_getservbyname (sh.string_to_pointer (a_name),
                                   sh.string_to_pointer (a_protocol))
         if p = default_pointer then
            -- service with this name does not exist
            raise_posix_error
         else
            set_members_from_struct (p, 0, a_protocol)
         end
      end

   make_from_port (a_port: INTEGER; a_protocol: STRING) is
      require
         valid_port: a_port >= 0
         -- make sure you have a `a_protocol' if necessary!
      local
         nport: INTEGER
         p: POINTER
      do
         nport := posix_htons (a_port)
         p := posix_getservbyport (nport, sh.string_to_pointer (a_protocol))
         set_members_from_struct (p, a_port, a_protocol)
      end


feature -- public state, all data in host byte order

   port: INTEGER
         -- port number

   name: STRING
         -- official service name

   aliases: ARRAY[STRING]
         -- alias list

   protocol: STRING
         -- protocol to use (udp/tcp)

   protocol_type: INTEGER
         -- SOCK_STREAM or SOCK_DGRAM


feature {NONE} -- set

   set_members_from_struct (p: POINTER; a_port: INTEGER; a_protocol: STRING) is
      require
         valid_struct: p /= default_pointer
      do
         if p = default_pointer then
            -- not found, guess?
            port := a_port
            name := "unknown"
            protocol := a_protocol
         create aliases.make (0, -1)
         else
            port := posix_ntohs (posix_servent_s_port (p))
            name := sh.pointer_to_string (posix_servent_s_name (p))
            protocol := sh.pointer_to_string (posix_servent_s_proto (p))
            aliases := ah.pointer_to_string_array (posix_servent_s_aliases (p))
         end
         protocol.to_lower
         if protocol.is_equal ("udp") then
            protocol_type := SOCK_DGRAM
         elseif protocol.is_equal ("tcp") then
            protocol_type := SOCK_STREAM
         else
            -- guess?, raise error?
            protocol_type := SOCK_STREAM
         end
      end


invariant

   valid_port: port >= 0
   valid_protocol:
      protocol /= Void and then
      protocol.is_equal ("tcp") or protocol.is_equal ("udp")
   valid_protocol_type:
      protocol_type = SOCK_STREAM or else
      protocol_type = SOCK_DGRAM

end
